@@ -65,7 +65,7 @@ class TestConnection(unittest.TestCase):
 
         Connection.HOST = "somedomainthatdoesntexist.org"
         Connection.PORT = 123456789
-        self.assertRaises(ConnectionFailure, Connection)
+        self.assertRaises(ConnectionFailure, Connection, connectTimeoutMS=600)
         self.assert_(Connection(self.host, self.port))
 
         Connection.HOST = self.host
@@ -74,7 +74,7 @@ class TestConnection(unittest.TestCase):
 
     def test_connect(self):
         self.assertRaises(ConnectionFailure, Connection,
-                          "somedomainthatdoesntexist.org")
+                          "somedomainthatdoesntexist.org", connectTimeoutMS=600)
         self.assertRaises(ConnectionFailure, Connection, self.host, 123456789)
 
         self.assert_(Connection(self.host, self.port))
@@ -361,6 +361,12 @@ class TestConnection(unittest.TestCase):
         self.assert_(isinstance(db.test.find_one(), dict))
         self.assertFalse(isinstance(db.test.find_one(), SON))
 
+    def test_timeouts(self):
+        conn = Connection(self.host, self.port, connectTimeoutMS=300)
+        self.assertEquals(0.3, conn._Connection__pool.conn_timeout)
+        conn = Connection(self.host, self.port, socketTimeoutMS=300)
+        self.assertEquals(0.3, conn._Connection__pool.net_timeout)
+
     def test_network_timeout(self):
         no_timeout = Connection(self.host, self.port)
         timeout = Connection(self.host, self.port, network_timeout=0.1)
@@ -404,7 +410,6 @@ class TestConnection(unittest.TestCase):
                 naive.pymongo_test.test.find_one()["x"])
 
     def test_ipv6(self):
-        self.assertRaises(AutoReconnect, Connection, 'foo')
         try:
             connection = Connection("[::1]")
         except:
@@ -424,15 +429,6 @@ class TestConnection(unittest.TestCase):
         self.assert_("pymongo_test" in dbs)
         self.assert_("pymongo_test_bernie" in dbs)
 
-    def test_autoreconnect(self):
-        def find_one(conn):
-            return conn.test.stuff.find_one()
-        # Simulate a temporary connection failure
-        c = Connection('foo', _connect=False)
-        self.assertRaises(AutoReconnect, find_one, c)
-        c._Connection__nodes = set([('localhost', 27017)])
-        self.assert_(find_one, c)
-
     def test_fsync_lock_unlock(self):
         c = get_connection()
         self.assertFalse(c.is_locked)
@@ -451,6 +447,35 @@ class TestConnection(unittest.TestCase):
             time.sleep(1)
         self.assertFalse(locked)
 
+    def test_contextlib(self):
+        if sys.version_info < (2, 6):
+            raise SkipTest()
+        import contextlib
+
+        conn = get_connection()
+        conn.pymongo_test.drop_collection("test")
+        conn.pymongo_test.test.insert({"foo": "bar"})
+        self.assertNotEqual(None, conn._Connection__pool.sock)
+        self.assertEqual(0, len(conn._Connection__pool.sockets))
+
+        # We need exec here because if the Python version is less than 2.6
+        # these with-statements won't even compile.
+        exec """
+with contextlib.closing(conn):
+    self.assertEquals("bar", conn.pymongo_test.test.find_one()["foo"])
+"""
+
+        self.assertEqual(None, conn._Connection__pool.sock)
+        self.assertEqual(0, len(conn._Connection__pool.sockets))
+
+        exec """
+with get_connection() as connection:
+    self.assertEquals("bar", connection.pymongo_test.test.find_one()["foo"])
+"""
+
+        self.assertEqual(None, connection._Connection__pool.sock)
+        self.assertEqual(0, len(connection._Connection__pool.sockets))
+
 
 if __name__ == "__main__":
     unittest.main()
